# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from fy_library.domain.mixin_models import BaseMixinModel


property mro_ordered_abstract_mixins: list[BaseMixinModel] using abstract_mixins_and_ordered_abstract_entities:
    property abstract_mixins
    property abstract_entities_ordering_index
fy"""

import abc
from functools import cached_property

from fy_library.domain.mixin_models import BaseMixinModel
from fy_library.mixins.property.entity_mixins.abstract_mixins.abc_fy import (
    AbstractMixins_PropertyMixin_ABC,
)
from fy_library.mixins.property.mro.mro_ordered_abstract_mixins.abc_fy import (
    MroOrderedAbstractMixins_PropertyMixin_ABC,
)
from fy_library.mixins.property.mro.ordered_abstract_entities.abc_fy import (
    AbstractEntitiesOrderingIndex_PropertyMixin_ABC,
)


# fy:start ===>>>
class MroOrderedAbstractMixins_UsingAbstractMixinsAndOrderedAbstractEntities_PropertyMixin(
    # Property Mixins
    AbstractEntitiesOrderingIndex_PropertyMixin_ABC,
    AbstractMixins_PropertyMixin_ABC,
    MroOrderedAbstractMixins_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _mro_ordered_abstract_mixins(self) -> list[BaseMixinModel]:
        # fy:end <<<===
        return sorted(
            self._abstract_mixins,
            key=lambda m: self._abstract_entities_ordering_index[m.entity_key],
        )



# fy:start ===>>>
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind


class MixinImportMap_UsingSetter_PropertyMixin:
    @property
    def _mixin_import_map(self) -> dict[tuple[ParsedFyPyFileKind, str], str]:
        return self.__mixin_import_map

    @_mixin_import_map.setter
    def _mixin_import_map(
        self, mixin_import_map: dict[tuple[ParsedFyPyFileKind, str], str]
    ) -> None:
        self.__mixin_import_map = mixin_import_map


# fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
flow generate_and_save_flow_compose_files__using_parsed_fy_py_files -> None:
    property mixin_import_map using setter
    property parsed_fy_py_files_map_by_key using setter
    property abstract_entities_ordering_index using setter
    property files_to_generate using setter
fy"""

from typing import Any

from fy_core.base.flow_base import FlowBase

from fy_library.domain.parsed_fy_py_file import (
    ParsedFyPyFile,
)
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.flows.generate_and_save_flow_compose_file__using_parsed_fy_py_file_fy import (
    GenerateAndSaveFlowComposeFile_UsingParsedFyPyFile_Flow,
)
from fy_library.mixins.property.imports_and_user_imports.mixin_import_map.using_setter import (
    MixinImportMap_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.mro.ordered_abstract_entities.using_setter import (
    AbstractEntitiesOrderingIndex_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files_map_by_key.using_setter import (
    ParsedFyPyFilesMapByKey_UsingSetter_PropertyMixin,
)

from fy_library.mixins.property.parsed_fy_py.files_to_generate.using_setter import (
    FilesToGenerate_UsingSetter_PropertyMixin,
)


# fy:start ===>>>
class GenerateAndSaveFlowComposeFiles_UsingParsedFyPyFiles_Flow(
    # Property Mixins
    MixinImportMap_UsingSetter_PropertyMixin,
    ParsedFyPyFilesMapByKey_UsingSetter_PropertyMixin,
    AbstractEntitiesOrderingIndex_UsingSetter_PropertyMixin,
    FilesToGenerate_UsingSetter_PropertyMixin,
    # Base
    FlowBase[None],
):
    def __init__(
        self,
        *args: Any,
        mixin_import_map: dict[tuple[ParsedFyPyFileKind, str], str],
        parsed_fy_py_files_map_by_key: dict[
            tuple[ParsedFyPyFileKind, str], ParsedFyPyFile
        ],
        abstract_entities_ordering_index: dict[tuple[ParsedFyPyFileKind, str], int],
        files_to_generate: list[ParsedFyPyFile],
        **kwargs: Any,
    ):
        self._mixin_import_map = mixin_import_map
        self._parsed_fy_py_files_map_by_key = parsed_fy_py_files_map_by_key
        self._abstract_entities_ordering_index = abstract_entities_ordering_index
        self._files_to_generate = files_to_generate
        super().__init__(*args, **kwargs)

    def __call__(self) -> None:
        # fy:end <<<===
        for parsed_fy_py_file in self._files_to_generate:
            GenerateAndSaveFlowComposeFile_UsingParsedFyPyFile_Flow(
                parsed_fy_py_file=parsed_fy_py_file,
                mixin_import_map=self._mixin_import_map,
                parsed_fy_py_files_map_by_key=self._parsed_fy_py_files_map_by_key,
                abstract_entities_ordering_index=self._abstract_entities_ordering_index,
            )()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
flow create_property_template_model__using_parsed_fy_py_file -> PropertyTemplateModel:
    property parsed_fy_py_file using setter
    property abstract_entities_ordering_index using setter
    property parsed_fy_py_files_map_by_key using setter
    property abstract_mixins using parsed_property_fy_py_file
    property mro_ordered_abstract_mixins using abstract_mixins_and_ordered_abstract_entities
fy"""

from typing import Any, cast

from fy_core.base.flow_base import FlowBase
from fy_library.domain.fy_py_template_models import PropertyTemplateModel
from fy_library.domain.mixin_models import AbstractPropertyModel
from fy_library.domain.parsed_fy_py_file import (
    ParsedFyPyFile,
    ParsedPropertyFyPyFile,
    ParsedAbstractPropertyFyPyFile,
)
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.mixins.property.entity_mixins.abstract_mixins.using_parsed_property_fy_py_file_fy import (
    AbstractMixins_UsingParsedPropertyFyPyFile_PropertyMixin,
)
from fy_library.mixins.property.mro.mro_ordered_abstract_mixins.new_parsed_fy_py_file_and_ordered_abstract_entities_fy import (
    MroOrderedAbstractMixins_UsingAbstractMixinsAndOrderedAbstractEntities_PropertyMixin,
)
from fy_library.mixins.property.mro.ordered_abstract_entities.using_setter import (
    AbstractEntitiesOrderingIndex_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_file.using_setter import (
    ParsedFyPyFile_UsingSetter_PropertyMixin,
)

from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files_map_by_key.using_setter import (
    ParsedFyPyFilesMapByKey_UsingSetter_PropertyMixin,
)


# fy:start ===>>>
class CreatePropertyTemplateModel_UsingParsedFyPyFile_Flow(
    # Property Mixins
    ParsedFyPyFile_UsingSetter_PropertyMixin,
    AbstractEntitiesOrderingIndex_UsingSetter_PropertyMixin,
    ParsedFyPyFilesMapByKey_UsingSetter_PropertyMixin,
    AbstractMixins_UsingParsedPropertyFyPyFile_PropertyMixin,
    MroOrderedAbstractMixins_UsingAbstractMixinsAndOrderedAbstractEntities_PropertyMixin,
    # Base
    FlowBase[PropertyTemplateModel],
):
    def __init__(
        self,
        *args: Any,
        parsed_fy_py_file: ParsedFyPyFile,
        abstract_entities_ordering_index: dict[tuple[ParsedFyPyFileKind, str], int],
        parsed_fy_py_files_map_by_key: dict[
            tuple[ParsedFyPyFileKind, str], ParsedFyPyFile
        ],
        **kwargs: Any,
    ):
        self._parsed_fy_py_file = parsed_fy_py_file
        self._abstract_entities_ordering_index = abstract_entities_ordering_index
        self._parsed_fy_py_files_map_by_key = parsed_fy_py_files_map_by_key
        super().__init__(*args, **kwargs)

    def __call__(self) -> PropertyTemplateModel:
        # fy:end <<<===
        parsed_property_fy_py_file = self._parsed_fy_py_file
        assert isinstance(parsed_property_fy_py_file, ParsedPropertyFyPyFile)

        def get_property_type(prop: AbstractPropertyModel) -> str:
            abstract_prop = cast(
                ParsedAbstractPropertyFyPyFile,
                self._parsed_fy_py_files_map_by_key[prop.entity_key],
            )
            if abstract_prop.generics_def:
                return prop.generics_impl
            return abstract_prop.property_type

        abstract_property_mixins: list[AbstractPropertyModel] = [
            AbstractPropertyModel(
                **abstract_property.model_dump(exclude={"property_type"}),
                property_type=get_property_type(
                    cast(AbstractPropertyModel, abstract_property)
                ),
            )
            for abstract_property in self._mro_ordered_abstract_mixins
        ]

        return PropertyTemplateModel(
            python_class_name=parsed_property_fy_py_file.python_class_name,
            property_name=parsed_property_fy_py_file.property_name,
            implementation_name=parsed_property_fy_py_file.implementation_name,
            abstract_property_mixins=abstract_property_mixins,
            generics_def=parsed_property_fy_py_file.generics_def,
            property_type=parsed_property_fy_py_file.property_type,
        )



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
property property_mixins_import: list[str] using property_mixins_and_mixin_import_map:
    property property_mixins
    property mixin_import_map
fy"""

import abc
from functools import cached_property

from fy_library.mixins.property.entity_mixins.property_mixins.abc_fy import (
    PropertyMixins_PropertyMixin_ABC,
)
from fy_library.mixins.property.imports_and_user_imports.mixin_import_map.abc_fy import (
    MixinImportMap_PropertyMixin_ABC,
)


# fy:start ===>>>
class PropertyMixinsImport_UsingPropertyMixinsAndMixinImportMap_PropertyMixin(
    # Property Mixins
    MixinImportMap_PropertyMixin_ABC,
    PropertyMixins_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _property_mixins_import(self) -> list[str]:
        # fy:end <<<===
        return [
            # property mixins
            self._mixin_import_map[property_mixin.entity_key]
            for property_mixin in self._property_mixins
        ]



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from fy_library.domain.fy_py_template_models import FlowTemplateModel


flow create_flow_template_model__using_parsed_fy_py_file_and_property_setters_template_models -> FlowTemplateModel:
    property parsed_fy_py_file using setter
    property parsed_fy_py_files_map_by_key using setter
    property property_mixins using template_model_properties
    property property_setter_mixins using property_mixins
    property property_constant_setter_mixins using property_mixins
    property parsed_fy_py_files using property_setter_mixins__mapped_to_abstract_property
fy"""

from typing import Any, cast

from fy_core.base.flow_base import FlowBase

from fy_library.domain.fy_py_template_models import (
    FlowTemplateModel,
)
from fy_library.domain.mixin_models import PropertyMixinModel, MethodMixinModel
from fy_library.domain.parsed_fy_py_file import (
    ParsedFyPyFile,
    ParsedFlowFyPyFile,
    ParsedAbstractPropertyFyPyFile,
    ParsedMethodFyPyFile,
    ParsedPropertyFyPyFile,
)
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.domain.python_entity_name import PythonEntityName
from fy_library.mixins.property.entity_mixins.property_mixins.using_template_model_properties_fy import (
    PropertyMixins_UsingTemplateModelProperties_PropertyMixin,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_file.using_setter import (
    ParsedFyPyFile_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files.using_property_setter_mixins__mapped_to_abstract_property__fy import (
    ParsedFyPyFiles_UsingPropertySetterMixins_MappedToAbstractProperty_PropertyMixin,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files_map_by_key.using_setter import (
    ParsedFyPyFilesMapByKey_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.property_setters.property_constant_setter_mixins.using_property_mixins_fy import (
    PropertyConstantSetterMixins_UsingPropertyMixins_PropertyMixin,
)
from fy_library.mixins.property.property_setters.property_setter_mixins.using_property_mixin_fy import (
    PropertySetterMixins_UsingPropertyMixins_PropertyMixin,
)


# fy:start ===>>>
class CreateFlowTemplateModel_UsingParsedFyPyFileAndPropertySettersTemplateModels_Flow(
    # Property Mixins
    ParsedFyPyFile_UsingSetter_PropertyMixin,
    ParsedFyPyFilesMapByKey_UsingSetter_PropertyMixin,
    PropertyMixins_UsingTemplateModelProperties_PropertyMixin,
    PropertySetterMixins_UsingPropertyMixins_PropertyMixin,
    PropertyConstantSetterMixins_UsingPropertyMixins_PropertyMixin,
    ParsedFyPyFiles_UsingPropertySetterMixins_MappedToAbstractProperty_PropertyMixin,
    # Base
    FlowBase[FlowTemplateModel],
):
    def __init__(
        self,
        *args: Any,
        parsed_fy_py_file: ParsedFyPyFile,
        parsed_fy_py_files_map_by_key: dict[
            tuple[ParsedFyPyFileKind, str], ParsedFyPyFile
        ],
        **kwargs: Any,
    ):
        self._parsed_fy_py_file = parsed_fy_py_file
        self._parsed_fy_py_files_map_by_key = parsed_fy_py_files_map_by_key
        super().__init__(*args, **kwargs)

    def __call__(self) -> FlowTemplateModel:
        # fy:end <<<===
        parsed_flow_fy_py_file = self._parsed_fy_py_file
        assert isinstance(parsed_flow_fy_py_file, ParsedFlowFyPyFile)
        declared_base_flow = PythonEntityName.from_snake_case(
            parsed_flow_fy_py_file.declared_base_flow
        )
        property_setters_names: set[str] = {
            cast(ParsedAbstractPropertyFyPyFile, p).abstract_property_name.snake_case
            for p in self._parsed_fy_py_files
        }.union(
            {p.property_name.snake_case for p in self._property_constant_setter_mixins}
        )

        def get_property_type(prop: PropertyMixinModel) -> str:
            if prop.entity_key in self._parsed_fy_py_files_map_by_key:
                return cast(
                    ParsedPropertyFyPyFile,
                    self._parsed_fy_py_files_map_by_key[prop.entity_key],
                ).property_type
            abstract_prop = cast(
                ParsedAbstractPropertyFyPyFile,
                self._parsed_fy_py_files_map_by_key[
                    (
                        ParsedFyPyFileKind.ABSTRACT_PROPERTY,
                        prop.property_name.snake_case,
                    )
                ],
            )
            if abstract_prop.generics_def:
                return prop.generics_impl
            return abstract_prop.property_type

        property_mixins: list[PropertyMixinModel] = [
            PropertyMixinModel(
                **prop.model_dump(exclude={"property_type"}),
                property_type=get_property_type(prop),
            )
            for prop in parsed_flow_fy_py_file.properties
        ]

        property_constant_setters: list[PropertyMixinModel] = [
            PropertyMixinModel(
                **prop.model_dump(exclude={"property_type"}),
                property_type=get_property_type(prop),
            )
            for prop in self._property_constant_setter_mixins
        ]

        method_mixins: list[MethodMixinModel] = [
            MethodMixinModel(
                **method.model_dump(exclude={"return_type"}),
                return_type=cast(
                    ParsedMethodFyPyFile,
                    self._parsed_fy_py_files_map_by_key[method.entity_key],
                ).return_type,
            )
            for method in parsed_flow_fy_py_file.methods
        ]

        property_mixins_in_use: list[PropertyMixinModel] = [
            property
            for property in property_mixins
            if f"self._{property.property_name.snake_case}"
            in self._parsed_fy_py_file.post_marker_file_content
        ]
        method_mixins_in_use: list[MethodMixinModel] = [
            method
            for method in method_mixins
            if f"self._{method.method_name.snake_case}"
            in self._parsed_fy_py_file.post_marker_file_content
        ]

        return FlowTemplateModel(
            python_class_name=parsed_flow_fy_py_file.python_class_name,
            flow_name=parsed_flow_fy_py_file.flow_name,
            generics_def=parsed_flow_fy_py_file.generics_def,
            declared_base_flow=PythonEntityName.from_pascal_case(
                f"{declared_base_flow.pascal_case}_BaseFlow"
            )
            if parsed_flow_fy_py_file.declared_base_flow != ""
            else None,
            declared_base_flow_generics_def=parsed_flow_fy_py_file.declared_base_flow_generics_def,
            return_type=parsed_flow_fy_py_file.return_type,
            properties=property_mixins,
            properties_without_setters=[
                p
                for p in parsed_flow_fy_py_file.properties
                if p.property_name.snake_case not in property_setters_names
            ],
            methods=method_mixins,
            property_setters=self._parsed_fy_py_files,
            property_constant_setters=property_constant_setters,
            property_mixins_in_use=property_mixins_in_use,
            method_mixins_in_use=method_mixins_in_use,
        )



# fy:start ===>>>
from typing import List
from fy_library.domain.mixin_models import AbstractPropertyModel


class AbstractPropertyMixins_UsingSetter_PropertyMixin:
    @property
    def _abstract_property_mixins(self) -> List[AbstractPropertyModel]:
        return self.__abstract_property_mixins

    @_abstract_property_mixins.setter
    def _abstract_property_mixins(
        self, abstract_property_mixins: List[AbstractPropertyModel]
    ) -> None:
        self.__abstract_property_mixins = abstract_property_mixins


# fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
property abstract_method_file_split: AbstractMethodFileSplitModel
fy"""

import abc
from pydantic import BaseModel


class AbstractMethodFileSplitModel(BaseModel):
    user_imports: str
    abstract_method_name: str
    generics_def: str
    arguments: str | None
    return_type: str


# fy:start ===>>>
class AbstractMethodFileSplit_PropertyMixin_ABC(abc.ABC):
    @property
    @abc.abstractmethod
    def _abstract_method_file_split(self) -> AbstractMethodFileSplitModel:
        raise NotImplementedError()
        # fy:end <<<===



# fy:start ===>>>
class BaseFlowImport_UsingSetter_PropertyMixin:
    @property
    def _base_flow_import(self) -> str:
        return self.__base_flow_import

    @_base_flow_import.setter
    def _base_flow_import(self, base_flow_import: str) -> None:
        self.__base_flow_import = base_flow_import
        # fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
from pathlib import Path
from typing import Literal, List
import abc
from pydantic import BaseModel, computed_field

from fy_library.constants import PROPERTY_SETTER_IMPLEMENTATION_NAME
from fy_library.domain.annotation_object import Annotation
from fy_library.domain.entity_key import entity_key
from fy_library.domain.mixin_models import (
    MethodMixinModel,
    AbstractMethodModel,
    AbstractPropertyModel,
    PropertyMixinModel,
)
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.domain.python_entity_name import PythonEntityName


class FyPyFileParts(BaseModel):
    pre_fy_code: str
    fy_code: str
    pre_marker_file_content: str
    post_marker_file_content: str


class ParsedFyPyFile(FyPyFileParts):
    file_type: ParsedFyPyFileKind
    file_path: Path
    user_imports: str
    python_class_name: PythonEntityName

    @property
    @abc.abstractmethod
    def entity_key(self) -> tuple[ParsedFyPyFileKind, str]:
        raise NotImplementedError()


class ParsedFlowFyPyFile(ParsedFyPyFile):
    file_type: Literal[ParsedFyPyFileKind.FLOW] = ParsedFyPyFileKind.FLOW
    flow_name: PythonEntityName
    generics_def: str
    declared_base_flow: str
    declared_base_flow_generics_def: str
    return_type: str
    properties: List[PropertyMixinModel]
    methods: List[MethodMixinModel]

    @computed_field
    @property
    def entity_key(self) -> tuple[ParsedFyPyFileKind, str]:
        return self.file_type, self.flow_name.snake_case


class ParsedBaseFlowFyPyFile(ParsedFyPyFile):
    file_type: Literal[ParsedFyPyFileKind.BASE_FLOW] = ParsedFyPyFileKind.BASE_FLOW
    base_flow_name: PythonEntityName
    generics_def: str
    declared_base_flow: str
    declared_base_flow_generics_def: str
    annotations: List[Annotation]
    return_type: str
    properties: List[PropertyMixinModel]
    methods: List[MethodMixinModel]
    abstract_property_mixins: List[AbstractPropertyModel]
    abstract_method_mixins: List[AbstractMethodModel]

    @computed_field
    @property
    def entity_key(self) -> tuple[ParsedFyPyFileKind, str]:
        return self.file_type, self.base_flow_name.snake_case


class ParsedMethodFyPyFile(ParsedFyPyFile):
    file_type: Literal[ParsedFyPyFileKind.METHOD] = ParsedFyPyFileKind.METHOD
    method_name: PythonEntityName
    abstract_property_mixins: List[AbstractPropertyModel]
    abstract_method_mixins: List[AbstractMethodModel]
    generics_def: str
    arguments: str | None
    implementation_name: PythonEntityName
    return_type: str

    @computed_field
    @property
    def entity_key(self) -> tuple[ParsedFyPyFileKind, str]:
        return entity_key(
            fy_py_kind=self.file_type,
            mixin_name__snake_case=self.method_name.snake_case,
            mixin_implementation_name__snake_case=self.implementation_name.snake_case,
        )


class ParsedAbstractMethodFyPyFile(ParsedFyPyFile):
    file_type: Literal[ParsedFyPyFileKind.ABSTRACT_METHOD] = (
        ParsedFyPyFileKind.ABSTRACT_METHOD
    )
    abstract_method_name: PythonEntityName
    generics_def: str
    arguments: str | None
    return_type: str

    @computed_field
    @property
    def entity_key(self) -> tuple[ParsedFyPyFileKind, str]:
        return self.file_type, self.abstract_method_name.snake_case


class ParsedAbstractPropertyFyPyFile(ParsedFyPyFile):
    file_type: Literal[ParsedFyPyFileKind.ABSTRACT_PROPERTY] = (
        ParsedFyPyFileKind.ABSTRACT_PROPERTY
    )
    abstract_property_name: PythonEntityName
    generics_def: str
    property_type: str

    @computed_field
    @property
    def entity_key(self) -> tuple[ParsedFyPyFileKind, str]:
        return self.file_type, self.abstract_property_name.snake_case


class ParsedPropertyFyPyFile(ParsedFyPyFile):
    file_type: Literal[ParsedFyPyFileKind.PROPERTY] = ParsedFyPyFileKind.PROPERTY
    property_name: PythonEntityName
    implementation_name: PythonEntityName
    abstract_property_mixins: List[AbstractPropertyModel]
    generics_def: str
    property_type: str

    @computed_field
    @property
    def entity_key(self) -> tuple[ParsedFyPyFileKind, str]:
        return entity_key(
            fy_py_kind=self.file_type,
            mixin_name__snake_case=self.property_name.snake_case,
            mixin_implementation_name__snake_case=self.implementation_name.snake_case,
        )


class PropertySetterFyPyFile(ParsedFyPyFile):
    file_type: Literal[ParsedFyPyFileKind.PROPERTY_SETTER] = (
        ParsedFyPyFileKind.PROPERTY_SETTER
    )
    property_name: PythonEntityName
    generics_def: str
    property_type: str
    implementation_name: PythonEntityName

    @computed_field
    @property
    def entity_key(self) -> tuple[ParsedFyPyFileKind, str]:
        return entity_key(
            fy_py_kind=self.file_type,
            mixin_name__snake_case=self.property_name.snake_case,
            mixin_implementation_name__snake_case=PROPERTY_SETTER_IMPLEMENTATION_NAME,
        )



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
method generate_and_save_flow_compose_code -> None using flow_compose_file_content:
    property flow_compose_file_content
    property flow_compose_file_name
fy"""

import abc

from fy_library.mixins.property.flow_compose_file.flow_compose_file_content.abc_fy import (
    FlowComposeFileContent_PropertyMixin_ABC,
)
from fy_library.mixins.property.flow_compose_file.flow_compose_file_name.abc_fy import (
    FlowComposeFileName_PropertyMixin_ABC,
)


# fy:start ===>>>
class GenerateAndSaveFlowComposeCode_UsingFlowComposeFileContent_MethodMixin(
    # Property Mixins
    FlowComposeFileContent_PropertyMixin_ABC,
    FlowComposeFileName_PropertyMixin_ABC,
    abc.ABC,
):
    def _generate_and_save_flow_compose_code(self) -> None:
        # fy:end <<<===
        flow_compose_file_content = self._flow_compose_file_content
        with open(
            file=self._flow_compose_file_name, mode="w", encoding="UTF-8"
        ) as setter_file:
            setter_file.write(flow_compose_file_content)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from fy_library.mixins.property.parsing_mixin_model.included_mixins.abc_fy import IncludedMixinsModel


property included_mixins: IncludedMixinsModel using mixin_models:
    property mixin_models
fy"""

import abc
from functools import cached_property


from fy_library.domain.mixin_models import MixinModelKind, BaseMixinModel
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.mixins.property.parsing_mixin_model.included_mixins.abc_fy import (
    IncludedMixins_PropertyMixin_ABC,
    IncludedMixinsModel,
)
from fy_library.mixins.property.parsing_mixin_model.mixin_models.abc_fy import (
    MixinModels_PropertyMixin_ABC,
)


# fy:start ===>>>
class IncludedMixins_UsingMixinModels_PropertyMixin(
    # Property Mixins
    IncludedMixins_PropertyMixin_ABC,
    MixinModels_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _included_mixins(self) -> IncludedMixinsModel:
        # fy:end <<<===
        check_for_duplicates_dict: dict[
            tuple[ParsedFyPyFileKind, str], BaseMixinModel
        ] = {}
        for mixin_model in self._mixin_models:
            if mixin_model.entity_key in check_for_duplicates_dict:
                raise AssertionError(
                    f"Duplicate found in {mixin_model.entity_key, mixin_model.kind.name}."
                )
            check_for_duplicates_dict[mixin_model.entity_key] = mixin_model

        included_mixins = IncludedMixinsModel(
            abstract_method_mixins=[
                mixin_model
                for mixin_model in self._mixin_models
                if mixin_model.kind == MixinModelKind.ABSTRACT_METHOD
            ],
            abstract_property_mixins=[
                mixin_model
                for mixin_model in self._mixin_models
                if mixin_model.kind == MixinModelKind.ABSTRACT_PROPERTY
            ],
            method_mixins=[
                mixin_model
                for mixin_model in self._mixin_models
                if mixin_model.kind == MixinModelKind.METHOD
            ],
            property_mixins=[
                mixin_model
                for mixin_model in self._mixin_models
                if mixin_model.kind == MixinModelKind.PROPERTY
            ],
        )

        return included_mixins



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy

from typing import List
from fy_library.domain.parsed_fy_py_file import ParsedFyPyFile


property parsed_fy_py_files_with_own_abstract_mixin: List[ParsedFyPyFile] using parsed_fy_py_files_map_by_key:
    property parsed_fy_py_files_map_by_key
fy"""

import abc
from functools import cached_property
from typing import List, cast

from fy_library.domain.mixin_models import (
    AbstractPropertyModel,
    MixinModelKind,
    AbstractMethodModel,
)
from fy_library.domain.parsed_fy_py_file import (
    ParsedFyPyFile,
    ParsedMethodFyPyFile,
    ParsedAbstractMethodFyPyFile,
    ParsedAbstractPropertyFyPyFile,
    ParsedPropertyFyPyFile,
)
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files_map_by_key.abc_fy import (
    ParsedFyPyFilesMapByKey_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files_with_own_abstract_mixin.abc_fy import (
    ParsedFyPyFilesWithOwnAbstractMixin_PropertyMixin_ABC,
)


# fy:start ===>>>
class ParsedFyPyFilesWithOwnAbstractMixin_UsingParsedFyPyFilesMapByKey_PropertyMixin(
    # Property Mixins
    ParsedFyPyFilesMapByKey_PropertyMixin_ABC,
    ParsedFyPyFilesWithOwnAbstractMixin_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _parsed_fy_py_files_with_own_abstract_mixin(self) -> List[ParsedFyPyFile]:
        # fy:end <<<===
        def with_own_abstract_mixin_if_exists(
            parsed_fy_py_file: ParsedFyPyFile,
        ) -> ParsedFyPyFile:
            match parsed_fy_py_file.file_type:
                case (
                    ParsedFyPyFileKind.FLOW
                    | ParsedFyPyFileKind.BASE_FLOW
                    | ParsedFyPyFileKind.ABSTRACT_PROPERTY
                    | ParsedFyPyFileKind.ABSTRACT_METHOD
                    | ParsedFyPyFileKind.PROPERTY_SETTER
                ):
                    return parsed_fy_py_file
                case ParsedFyPyFileKind.METHOD:
                    parsed_method_fy_py_file = cast(
                        ParsedMethodFyPyFile, parsed_fy_py_file
                    )
                    own_abstract_method_entity_key = (
                        ParsedFyPyFileKind.ABSTRACT_METHOD,
                        parsed_method_fy_py_file.method_name.snake_case,
                    )
                    if (
                        own_abstract_method_entity_key
                    ) not in self._parsed_fy_py_files_map_by_key:
                        return parsed_fy_py_file

                    parsed_abstract_method_fy_py_file = cast(
                        ParsedAbstractMethodFyPyFile,
                        self._parsed_fy_py_files_map_by_key[
                            own_abstract_method_entity_key
                        ],
                    )

                    if any(
                        abstract_method_mixin.python_class_name.pascal_case
                        == parsed_abstract_method_fy_py_file.python_class_name.pascal_case
                        for abstract_method_mixin in parsed_method_fy_py_file.abstract_method_mixins
                    ):
                        return parsed_fy_py_file

                    return ParsedMethodFyPyFile.model_validate(
                        {
                            **parsed_method_fy_py_file.model_dump(),
                            "abstract_method_mixins": parsed_method_fy_py_file.abstract_method_mixins
                            + [
                                AbstractMethodModel(
                                    python_class_name=parsed_abstract_method_fy_py_file.python_class_name,
                                    kind=MixinModelKind.ABSTRACT_METHOD,
                                    method_name=parsed_abstract_method_fy_py_file.abstract_method_name,
                                    generics_impl=(
                                        parsed_method_fy_py_file.generics_def
                                        if parsed_abstract_method_fy_py_file.generics_def
                                        != ""
                                        else ""
                                    ),
                                    return_type="",
                                )
                            ],
                        }
                    )
                case ParsedFyPyFileKind.PROPERTY:
                    parsed_property_fy_py_file = cast(
                        ParsedPropertyFyPyFile, parsed_fy_py_file
                    )
                    own_abstract_property_entity_key = (
                        ParsedFyPyFileKind.ABSTRACT_PROPERTY,
                        parsed_property_fy_py_file.property_name.snake_case,
                    )
                    if (
                        own_abstract_property_entity_key
                    ) not in self._parsed_fy_py_files_map_by_key:
                        return parsed_fy_py_file

                    parsed_abstract_property_fy_py_file = cast(
                        ParsedAbstractPropertyFyPyFile,
                        self._parsed_fy_py_files_map_by_key[
                            own_abstract_property_entity_key
                        ],
                    )

                    if any(
                        abstract_mixin.python_class_name.pascal_case
                        == parsed_abstract_property_fy_py_file.python_class_name.pascal_case
                        for abstract_mixin in parsed_property_fy_py_file.abstract_property_mixins
                    ):
                        return parsed_fy_py_file

                    return ParsedPropertyFyPyFile.model_validate(
                        {
                            **parsed_property_fy_py_file.model_dump(),
                            "abstract_property_mixins": parsed_property_fy_py_file.abstract_property_mixins
                            + [
                                AbstractPropertyModel(
                                    python_class_name=parsed_abstract_property_fy_py_file.python_class_name,
                                    kind=MixinModelKind.ABSTRACT_PROPERTY,
                                    property_name=parsed_property_fy_py_file.property_name,
                                    generics_impl=(
                                        parsed_property_fy_py_file.generics_def
                                        or parsed_property_fy_py_file.property_type
                                    )
                                    if parsed_abstract_property_fy_py_file.generics_def
                                    != ""
                                    else "",
                                    property_type="",
                                )
                            ],
                        }
                    )

        return [
            with_own_abstract_mixin_if_exists(
                parsed_fy_py_file=self._parsed_fy_py_files_map_by_key[
                    parsed_fy_py_file_key
                ]
            )
            for parsed_fy_py_file_key in self._parsed_fy_py_files_map_by_key
        ]



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
property files_to_generate: list[ParsedFyPyFile] using parsed_fy_py_file__and__folder_to_generate:
    property parsed_fy_py_files
    property folder_to_generate
fy"""

import abc
from functools import cached_property

from fy_library.domain.parsed_fy_py_file import ParsedFyPyFile
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.mixins.property.fy_file.folder_to_generate.abc_fy import (
    FolderToGenerate_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_fy_py.files_to_generate.abc_fy import (
    FilesToGenerate_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files.abc_fy import (
    ParsedFyPyFiles_PropertyMixin_ABC,
)


# fy:start ===>>>
class FilesToGenerate_UsingParsedFyPyFile_And_FolderToGenerate_PropertyMixin(
    # Property Mixins
    FilesToGenerate_PropertyMixin_ABC,
    FolderToGenerate_PropertyMixin_ABC,
    ParsedFyPyFiles_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _files_to_generate(self) -> list[ParsedFyPyFile]:
        # fy:end <<<===
        return [
            parsed_file
            for parsed_file in self._parsed_fy_py_files
            if parsed_file.file_path.parent.is_relative_to(self._folder_to_generate)
            and parsed_file.file_type
            in {
                ParsedFyPyFileKind.FLOW,
                ParsedFyPyFileKind.METHOD,
                ParsedFyPyFileKind.PROPERTY,
            }
        ]



# fy:start ===>>>
from typing import List


class MixinLines_UsingSetter_PropertyMixin:
    @property
    def _mixin_lines(self) -> List[str]:
        return self.__mixin_lines

    @_mixin_lines.setter
    def _mixin_lines(self, mixin_lines: List[str]) -> None:
        self.__mixin_lines = mixin_lines
        # fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from typing import List
from fy_library.domain.mixin_models import PropertyMixinModel


property property_mixins: List[PropertyMixinModel] using template_model_properties:
    property parsed_fy_py_file
fy"""

import abc
from functools import cached_property
from typing import List

from fy_library.domain.mixin_models import PropertyMixinModel
from fy_library.mixins.property.entity_mixins.property_mixins.abc_fy import (
    PropertyMixins_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_file.abc_fy import (
    ParsedFyPyFile_PropertyMixin_ABC,
)


# fy:start ===>>>
class PropertyMixins_UsingTemplateModelProperties_PropertyMixin(
    # Property Mixins
    ParsedFyPyFile_PropertyMixin_ABC,
    PropertyMixins_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _property_mixins(self) -> List[PropertyMixinModel]:
        # fy:end <<<===
        assert hasattr(self._parsed_fy_py_file, "properties")

        return self._parsed_fy_py_file.properties



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
from enum import Enum


class ParsedFyPyFileKind(str, Enum):
    FLOW = "flow"
    BASE_FLOW = "base_flow"
    METHOD = "method"
    ABSTRACT_METHOD = "abstract_method"
    ABSTRACT_PROPERTY = "abstract_property"
    PROPERTY = "property"
    PROPERTY_SETTER = "property_setter"



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
property user_imports_from_mixins: list[str] using property_setter_mixins:
    property parsed_fy_py_files_map_by_key
    property property_setter_mixins
fy"""

import abc
from functools import cached_property
from typing import Set

from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files_map_by_key.abc_fy import (
    ParsedFyPyFilesMapByKey_PropertyMixin_ABC,
)
from fy_library.mixins.property.property_setters.property_setter_mixins.abc_fy import (
    PropertySetterMixins_PropertyMixin_ABC,
)


# fy:start ===>>>
class UserImportsFromMixins_UsingPropertySetterMixins_PropertyMixin(
    # Property Mixins
    ParsedFyPyFilesMapByKey_PropertyMixin_ABC,
    PropertySetterMixins_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _user_imports_from_mixins(self) -> list[str]:
        # fy:end <<<===
        user_imports_split = self._parsed_fy_py_files_map_by_key
        user_imports: Set[str] = set()
        for property_setter in self._property_setter_mixins:
            if property_setter.generics_impl != "":
                # Generic property setters that have generics_impl
                # will have an import defined in the flow.
                continue
            user_imports.update(
                [
                    user_import
                    for user_import in user_imports_split[
                        ParsedFyPyFileKind.ABSTRACT_PROPERTY,
                        property_setter.property_name.snake_case,
                    ].user_imports.split("\n")
                    if user_import != ""
                ]
            )

        return list(user_imports)



# fy:start ===>>>
from fy_library.domain.parsed_fy_py_file import ParsedFyPyFile


class FilesToGenerate_UsingSetter_PropertyMixin:
    @property
    def _files_to_generate(self) -> list[ParsedFyPyFile]:
        return self.__files_to_generate

    @_files_to_generate.setter
    def _files_to_generate(self, files_to_generate: list[ParsedFyPyFile]) -> None:
        self.__files_to_generate = files_to_generate


# fy:end <<<===



from pathlib import Path

from fy_library.flows.fy_py_main_fy import FyPy_Main_Flow


def fy(
    folder_to_parse: Path,
    project_root_folder: Path,
) -> None:
    FyPy_Main_Flow(
        folder_to_parse=folder_to_parse,
        project_root_folder=(
            Path(project_root_folder) if project_root_folder is not None else Path.cwd()
        ),
    )()



# fy:start ===>>>
class MixinLine_UsingSetter_PropertyMixin:
    @property
    def _mixin_line(self) -> str:
        return self.__mixin_line

    @_mixin_line.setter
    def _mixin_line(self, mixin_line: str) -> None:
        self.__mixin_line = mixin_line


# fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind


def entity_key(
    fy_py_kind: ParsedFyPyFileKind,
    mixin_name__snake_case: str,
    mixin_implementation_name__snake_case: str,
) -> tuple[ParsedFyPyFileKind, str]:
    return (
        fy_py_kind,
        f"{mixin_name__snake_case}.{mixin_implementation_name__snake_case}",
    )



# fy:start ===>>>
from typing import List
from fy_library.domain.parsed_fy_py_file import PropertySetterFyPyFile


class RequiredPropertySettersFyPy_UsingSetter_PropertyMixin:
    @property
    def _required_property_setters_fy_py(self) -> List[PropertySetterFyPyFile]:
        return self.__required_property_setters_fy_py

    @_required_property_setters_fy_py.setter
    def _required_property_setters_fy_py(
        self, required_property_setters_fy_py: List[PropertySetterFyPyFile]
    ) -> None:
        self.__required_property_setters_fy_py = required_property_setters_fy_py


# fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from typing import List



property import_base_flow: List[str] using declared_base_flow_name:
    property declared_base_flow_name
    property mixin_import_map
fy"""

import abc
from functools import cached_property
from typing import List

from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.mixins.property.imports_and_user_imports.mixin_import_map.abc_fy import (
    MixinImportMap_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_entity_fy_py_file.declared_base_flow_name.abc_fy import (
    DeclaredBaseFlowName_PropertyMixin_ABC,
)


# fy:start ===>>>
class ImportBaseFlow_UsingDeclaredBaseFlowName_PropertyMixin(
    # Property Mixins
    DeclaredBaseFlowName_PropertyMixin_ABC,
    MixinImportMap_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _import_base_flow(self) -> List[str]:
        # fy:end <<<===
        if self._declared_base_flow_name == "":
            return []
        return [
            self._mixin_import_map[
                ParsedFyPyFileKind.BASE_FLOW, self._declared_base_flow_name
            ]
        ]



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
property required_setters: dict[str, PropertySetterFyPyFile] using parsed_fy_py_files:
    property parsed_fy_py_files
    property parsed_fy_py_files_map_by_key
fy"""

import abc
from functools import cached_property
from typing import List, cast

from fy_library.constants import (
    GENERATED_PROPERTY_FILE_IMPLEMENTATION_NAMES,
    PROPERTY_SETTER_IMPLEMENTATION_NAME,
)
from fy_library.domain.entity_key import entity_key
from fy_library.domain.fy_py_template_models import TemporaryBaseTemplateModel
from fy_library.domain.mixin_models import PropertyMixinModel
from fy_library.domain.parsed_fy_py_file import (
    PropertySetterFyPyFile,
    ParsedFyPyFile,
    ParsedAbstractPropertyFyPyFile,
)
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files.abc_fy import (
    ParsedFyPyFiles_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files_map_by_key.abc_fy import (
    ParsedFyPyFilesMapByKey_PropertyMixin_ABC,
)
from fy_library.mixins.property.property_setters.required_setters.abc_fy import (
    RequiredSetters_PropertyMixin_ABC,
)


# fy:start ===>>>
class RequiredSetters_UsingParsedFyPyFiles_PropertyMixin(
    # Property Mixins
    ParsedFyPyFiles_PropertyMixin_ABC,
    ParsedFyPyFilesMapByKey_PropertyMixin_ABC,
    RequiredSetters_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _required_setters(self) -> dict[str, PropertySetterFyPyFile]:
        # fy:end <<<===
        def get_properties(
            parsed_fy_py_file: ParsedFyPyFile,
        ) -> List[PropertyMixinModel]:
            assert hasattr(parsed_fy_py_file, "properties")
            return cast(List[PropertyMixinModel], parsed_fy_py_file.properties)

        required_setters = {
            flow_property.property_name.snake_case: PropertySetterFyPyFile(
                pre_fy_code="",
                fy_code="",
                pre_marker_file_content="",
                post_marker_file_content="",
                file_path=self._parsed_fy_py_files_map_by_key[
                    ParsedFyPyFileKind.ABSTRACT_PROPERTY,
                    flow_property.property_name.snake_case,
                ].file_path.with_name("using_setter.py"),
                user_imports=self._parsed_fy_py_files_map_by_key[
                    ParsedFyPyFileKind.ABSTRACT_PROPERTY,
                    flow_property.property_name.snake_case,
                ].user_imports,
                python_class_name=flow_property.python_class_name,
                generics_def=cast(
                    ParsedAbstractPropertyFyPyFile,
                    self._parsed_fy_py_files_map_by_key[
                        ParsedFyPyFileKind.ABSTRACT_PROPERTY,
                        flow_property.property_name.snake_case,
                    ],
                ).generics_def,
                property_type=cast(
                    ParsedAbstractPropertyFyPyFile,
                    self._parsed_fy_py_files_map_by_key[
                        ParsedFyPyFileKind.ABSTRACT_PROPERTY,
                        flow_property.property_name.snake_case,
                    ],
                ).property_type,
                property_name=flow_property.property_name,
                implementation_name=flow_property.implementation_name,
                template_model=TemporaryBaseTemplateModel(
                    python_class_name=flow_property.python_class_name,
                    entity_key_value=entity_key(
                        fy_py_kind=ParsedFyPyFileKind.PROPERTY_SETTER,
                        mixin_name__snake_case=flow_property.property_name.snake_case,
                        mixin_implementation_name__snake_case=PROPERTY_SETTER_IMPLEMENTATION_NAME,
                    ),
                ),
            )
            for parsed_fy_py_file in self._parsed_fy_py_files
            if parsed_fy_py_file.file_type
            in {ParsedFyPyFileKind.FLOW, ParsedFyPyFileKind.BASE_FLOW}
            for flow_property in get_properties(parsed_fy_py_file)
            if (
                flow_property.implementation_name.snake_case
                in GENERATED_PROPERTY_FILE_IMPLEMENTATION_NAMES
                and (
                    flow_property.entity_key[0],
                    f"{flow_property.property_name.snake_case}.setter",
                )
                not in self._parsed_fy_py_files_map_by_key
            )
        }

        return required_setters



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
property flow_compose_file_content: str using parsed_fy_py_file:
    property parsed_fy_py_file
    property generated_flow_compose_code
fy"""

import abc
from functools import cached_property
from typing import cast

from fy_library.domain.parsed_fy_py_file import (
    ParsedPropertyFyPyFile,
    ParsedMethodFyPyFile,
    ParsedFlowFyPyFile,
)
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.mixins.property.flow_compose_file.flow_compose_file_content.abc_fy import (
    FlowComposeFileContent_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_file.abc_fy import (
    ParsedFyPyFile_PropertyMixin_ABC,
)

from fy_library.mixins.property.templates.generated_flow_compose_code.abc_fy import (
    GeneratedFlowComposeCode_PropertyMixin_ABC,
)


# fy:start ===>>>
class FlowComposeFileContent_UsingParsedFyPyFile_PropertyMixin(
    # Property Mixins
    FlowComposeFileContent_PropertyMixin_ABC,
    GeneratedFlowComposeCode_PropertyMixin_ABC,
    ParsedFyPyFile_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _flow_compose_file_content(self) -> str:
        # fy:end <<<===
        replaced_file_content = self._parsed_fy_py_file.post_marker_file_content
        method_mixin_names: list[str] = []
        property_mixin_names: list[str] = []
        match self._parsed_fy_py_file.file_type:
            case ParsedFyPyFileKind.PROPERTY:
                property_mixin_names = list(
                    map(
                        lambda p: p.property_name.snake_case,
                        cast(
                            ParsedPropertyFyPyFile, self._parsed_fy_py_file
                        ).abstract_property_mixins,
                    )
                )
            case ParsedFyPyFileKind.METHOD:
                property_mixin_names = list(
                    map(
                        lambda p: p.property_name.snake_case,
                        cast(
                            ParsedMethodFyPyFile, self._parsed_fy_py_file
                        ).abstract_property_mixins,
                    )
                )
                method_mixin_names = list(
                    map(
                        lambda p: p.method_name.snake_case,
                        cast(
                            ParsedMethodFyPyFile, self._parsed_fy_py_file
                        ).abstract_method_mixins,
                    )
                )
            case ParsedFyPyFileKind.FLOW:
                property_mixin_names = list(
                    map(
                        lambda p: p.property_name.snake_case,
                        cast(ParsedFlowFyPyFile, self._parsed_fy_py_file).properties,
                    )
                )
                method_mixin_names = list(
                    map(
                        lambda p: p.method_name.snake_case,
                        cast(ParsedFlowFyPyFile, self._parsed_fy_py_file).methods,
                    )
                )

        for mixin_name in property_mixin_names:
            self_mixin = f"self._{mixin_name}"
            function_call = f"{mixin_name}()"
            replaced_file_content = replaced_file_content.replace(
                self_mixin, function_call
            )

        for mixin_name in method_mixin_names:
            self_mixin = f"self._{mixin_name}"
            function_call = mixin_name
            replaced_file_content = replaced_file_content.replace(
                self_mixin, function_call
            )

        stripped_tab_lines = map(
            lambda line: line[4:],
            replaced_file_content.splitlines(),
        )
        stripped_tab_content = "\n".join(stripped_tab_lines)
        return f"{self._generated_flow_compose_code}\n{stripped_tab_content}\n"



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from typing import List
from fy_library.domain.parsed_fy_py_file import ParsedFyPyFile


property parsed_fy_py_files: List[ParsedFyPyFile] using property_setter_mixins__mapped_to_abstract_property:
    property property_setter_mixins
    property parsed_fy_py_files_map_by_key
fy"""

import abc
from functools import cached_property
from typing import List, cast

from fy_library.domain.mixin_models import PropertyMixinModel
from fy_library.domain.parsed_fy_py_file import (
    ParsedFyPyFile,
    ParsedAbstractPropertyFyPyFile,
)
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files.abc_fy import (
    ParsedFyPyFiles_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files_map_by_key.abc_fy import (
    ParsedFyPyFilesMapByKey_PropertyMixin_ABC,
)
from fy_library.mixins.property.property_setters.property_setter_mixins.abc_fy import (
    PropertySetterMixins_PropertyMixin_ABC,
)


# fy:start ===>>>
class ParsedFyPyFiles_UsingPropertySetterMixins_MappedToAbstractProperty_PropertyMixin(
    # Property Mixins
    ParsedFyPyFiles_PropertyMixin_ABC,
    ParsedFyPyFilesMapByKey_PropertyMixin_ABC,
    PropertySetterMixins_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _parsed_fy_py_files(self) -> List[ParsedFyPyFile]:
        # fy:end <<<===
        def replace_generics_impl_type(
            parsed_abstract_property: ParsedAbstractPropertyFyPyFile,
            property_mixin: PropertyMixinModel,
        ) -> ParsedAbstractPropertyFyPyFile:
            if property_mixin.generics_impl == "":
                return parsed_abstract_property
            return ParsedAbstractPropertyFyPyFile(
                **parsed_abstract_property.model_dump(exclude={"property_type"}),
                property_type=property_mixin.generics_impl,
            )

        return [
            replace_generics_impl_type(
                parsed_abstract_property=cast(
                    ParsedAbstractPropertyFyPyFile,
                    self._parsed_fy_py_files_map_by_key[
                        ParsedFyPyFileKind.ABSTRACT_PROPERTY,
                        property_setter.property_name.snake_case,
                    ],
                ),
                property_mixin=property_setter,
            )
            for property_setter in self._property_setter_mixins
        ]



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from fy_library.domain.parsed_fy_py_file import ParsedFyPyFile


flow parse_fy_py_file -> ParsedFyPyFile:
    property fy_py_file_to_parse using setter
    property fy_py_file_parts using fy_file_to_parse_docstring
    property pre_fy_code using fy_py_file_parts
    property fy_code using fy_py_file_parts
    property post_marker_file_content using fy_py_file_parts
    property pre_marker_file_content using fy_py_file_parts
    property fy_file_kind using fy_code
fy"""

from pathlib import Path
from typing import Any

from fy_core.base.flow_base import FlowBase
from fy_library.domain.parsed_fy_py_file import ParsedFyPyFile
from fy_library.domain.parsed_fy_py_file_kind import ParsedFyPyFileKind
from fy_library.flows.parse_abstract_method_fy_code_fy import (
    ParseAbstractMethodFyCode_Flow,
)
from fy_library.flows.parse_abstract_property_fy_code_fy import (
    ParseAbstractPropertyFyCode_Flow,
)
from fy_library.flows.parse_base_flow_fy_code_fy import ParseBaseFlowFyCode_Flow
from fy_library.flows.parse_flow_fy_code_fy import ParseFlowFyCode_Flow
from fy_library.flows.parse_method_fy_code_fy import ParseMethodFyCode_Flow
from fy_library.flows.parse_property_fy_code_fy import ParsePropertyFyCode_Flow
from fy_library.mixins.property.fy_file.fy_code.using_fy_py_file_parts_fy import (
    FyCode_UsingFyPyFileParts_PropertyMixin,
)
from fy_library.mixins.property.fy_file.fy_file_kind.using_fy_code_fy import (
    FyFileKind_UsingFyCode_PropertyMixin,
)
from fy_library.mixins.property.fy_file.post_marker_file_content.using_fy_py_file_parts_fy import (
    PostMarkerFileContent_UsingFyPyFileParts_PropertyMixin,
)
from fy_library.mixins.property.fy_file.pre_fy_code.using_fy_py_file_parts_fy import (
    PreFyCode_UsingFyPyFileParts_PropertyMixin,
)
from fy_library.mixins.property.fy_file.pre_marker_file_content.using_fy_py_file_parts_fy import (
    PreMarkerFileContent_UsingFyPyFileParts_PropertyMixin,
)
from fy_library.mixins.property.fy_py_file.fy_py_file_parts.using_fy_file_to_parse_docstring_fy import (
    FyPyFileParts_UsingFyFileToParseDocstring_PropertyMixin,
)
from fy_library.mixins.property.fy_py_file.fy_py_file_to_parse.using_setter import (
    FyPyFileToParse_UsingSetter_PropertyMixin,
)


# fy:start ===>>>
class ParseFyPyFile_Flow(
    # Property Mixins
    FyPyFileToParse_UsingSetter_PropertyMixin,
    FyPyFileParts_UsingFyFileToParseDocstring_PropertyMixin,
    PreFyCode_UsingFyPyFileParts_PropertyMixin,
    FyCode_UsingFyPyFileParts_PropertyMixin,
    PostMarkerFileContent_UsingFyPyFileParts_PropertyMixin,
    PreMarkerFileContent_UsingFyPyFileParts_PropertyMixin,
    FyFileKind_UsingFyCode_PropertyMixin,
    # Base
    FlowBase[ParsedFyPyFile],
):
    def __init__(
        self,
        *args: Any,
        fy_py_file_to_parse: Path,
        **kwargs: Any,
    ):
        self._fy_py_file_to_parse = fy_py_file_to_parse
        super().__init__(*args, **kwargs)

    def __call__(self) -> ParsedFyPyFile:
        # fy:end <<<===
        parse_fy_code: FlowBase[ParsedFyPyFile]
        match self._fy_file_kind:
            case ParsedFyPyFileKind.FLOW:
                parse_fy_code = ParseFlowFyCode_Flow(
                    pre_fy_code=self._pre_fy_code,
                    fy_code=self._fy_code,
                    pre_marker_file_content=self._pre_marker_file_content,
                    post_marker_file_content=self._post_marker_file_content,
                    fy_py_file_to_parse=self._fy_py_file_to_parse,
                )
            case ParsedFyPyFileKind.BASE_FLOW:
                parse_fy_code = ParseBaseFlowFyCode_Flow(
                    pre_fy_code=self._pre_fy_code,
                    fy_code=self._fy_code,
                    pre_marker_file_content=self._pre_marker_file_content,
                    post_marker_file_content=self._post_marker_file_content,
                    fy_py_file_to_parse=self._fy_py_file_to_parse,
                )
            case ParsedFyPyFileKind.METHOD:
                parse_fy_code = ParseMethodFyCode_Flow(
                    pre_fy_code=self._pre_fy_code,
                    fy_code=self._fy_code,
                    pre_marker_file_content=self._pre_marker_file_content,
                    post_marker_file_content=self._post_marker_file_content,
                    fy_py_file_to_parse=self._fy_py_file_to_parse,
                )
            case ParsedFyPyFileKind.ABSTRACT_METHOD:
                parse_fy_code = ParseAbstractMethodFyCode_Flow(
                    pre_fy_code=self._pre_fy_code,
                    fy_code=self._fy_code,
                    pre_marker_file_content=self._pre_marker_file_content,
                    post_marker_file_content=self._post_marker_file_content,
                    fy_py_file_to_parse=self._fy_py_file_to_parse,
                )
            case ParsedFyPyFileKind.ABSTRACT_PROPERTY:
                parse_fy_code = ParseAbstractPropertyFyCode_Flow(
                    pre_fy_code=self._pre_fy_code,
                    fy_code=self._fy_code,
                    pre_marker_file_content=self._pre_marker_file_content,
                    post_marker_file_content=self._post_marker_file_content,
                    fy_py_file_to_parse=self._fy_py_file_to_parse,
                )
            case ParsedFyPyFileKind.PROPERTY:
                parse_fy_code = ParsePropertyFyCode_Flow(
                    pre_fy_code=self._pre_fy_code,
                    fy_code=self._fy_code,
                    pre_marker_file_content=self._pre_marker_file_content,
                    post_marker_file_content=self._post_marker_file_content,
                    fy_py_file_to_parse=self._fy_py_file_to_parse,
                )
            case _:
                raise NotImplementedError(
                    f"No parser implemented for file kind '{self._fy_file_kind}' in file '{self._fy_py_file_to_parse}'"
                )

        return parse_fy_code()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from fy_library.domain.mixin_models import PropertyMixinModel


property optional_property_mixin_model: PropertyMixinModel | None using mixin_line:
    property mixin_line
fy"""

import abc
import re
from functools import cached_property
from typing import Final

from fy_library.constants import (
    FY_ENTITY_REGEX_STRING,
    PYTHON_MULTI_ENTITY_REGEX_STRING,
    PROPERTY_CONSTANT_IMPLEMENTATION_NAME,
    PROPERTY_SETTER_IMPLEMENTATION_NAME,
)
from fy_library.domain.mixin_models import MixinModelKind, PropertyMixinModel
from fy_library.domain.python_entity_name import PythonEntityName
from fy_library.mixins.property.optional_mixin_models.optional_property_mixin_model.abc_fy import (
    OptionalPropertyMixinModel_PropertyMixin_ABC,
)
from fy_library.mixins.property.parsing_mixin_model.mixin_line.abc_fy import (
    MixinLine_PropertyMixin_ABC,
)

_FLOW_PROPERTY_REGEX: Final = re.compile(
    rf"^\s+property\s+(?P<property_name>{FY_ENTITY_REGEX_STRING})\s+"
    rf"using\s+(?P<implementation_name>(?!constant\b){FY_ENTITY_REGEX_STRING})?"
    rf"(?:\[(?P<generics_impl>{PYTHON_MULTI_ENTITY_REGEX_STRING})])?"
    rf"(?:constant(?:\[(?P<constant_generics_impl>{PYTHON_MULTI_ENTITY_REGEX_STRING})])?\((?P<constant_value>.*)\))?"
    r"\s*$"
)


# fy:start ===>>>
class OptionalPropertyMixinModel_UsingMixinLine_PropertyMixin(
    # Property Mixins
    MixinLine_PropertyMixin_ABC,
    OptionalPropertyMixinModel_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _optional_property_mixin_model(self) -> PropertyMixinModel | None:
        # fy:end <<<===
        flow_property_fy_search = _FLOW_PROPERTY_REGEX.search(self._mixin_line)

        if flow_property_fy_search is None:
            return None

        property_name: PythonEntityName = PythonEntityName.from_snake_case(
            flow_property_fy_search.group("property_name")
        )
        maybe_constant_value = flow_property_fy_search.group("constant_value")
        maybe_implementation_name = flow_property_fy_search.group("implementation_name")

        assert maybe_implementation_name is not None or maybe_constant_value is not None

        python_class_implementation_name = PythonEntityName.from_snake_case(
            flow_property_fy_search.group("implementation_name")
            or PROPERTY_SETTER_IMPLEMENTATION_NAME
        )

        python_class_name = PythonEntityName.from_pascal_case(
            f"{ property_name.pascal_case }_Using{ python_class_implementation_name.pascal_case }_PropertyMixin"
        )

        implementation_name: PythonEntityName = PythonEntityName.from_snake_case(
            flow_property_fy_search.group("implementation_name")
            or PROPERTY_CONSTANT_IMPLEMENTATION_NAME
        )

        assert (
            flow_property_fy_search.group("generics_impl") is None
            or flow_property_fy_search.group("constant_generics_impl") is None
        )

        generics_impl = (
            flow_property_fy_search.group("generics_impl")
            if implementation_name.snake_case != PROPERTY_CONSTANT_IMPLEMENTATION_NAME
            else flow_property_fy_search.group("constant_generics_impl")
        )

        return PropertyMixinModel(
            python_class_name=python_class_name,
            kind=MixinModelKind.PROPERTY,
            property_name=property_name,
            implementation_name=implementation_name,
            generics_impl=generics_impl or "",
            constant_value=flow_property_fy_search.group("constant_value") or "",
            property_type="",
        )



# fy:start ===>>>
from pathlib import Path


class FolderToGenerate_UsingSetter_PropertyMixin:
    @property
    def _folder_to_generate(self) -> Path:
        return self.__folder_to_generate

    @_folder_to_generate.setter
    def _folder_to_generate(self, folder_to_generate: Path) -> None:
        self.__folder_to_generate = folder_to_generate


# fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from pathlib import Path
from typing import Any


flow flow_compose__main -> None:
    property folder_to_parse using setter
    property project_root_folder using setter
    property folder_to_generate using setter
    property fy_py_files_to_parse using files_discovery
    property parsed_fy_py_files using fy_py_parser
    property parsed_fy_py_files_map_by_key using parsed_fy_py_files
    property parsed_fy_py_files_with_own_abstract_mixin using parsed_fy_py_files_map_by_key
    property abstract_entities_ordering_index using parsed_fy_py_files_map_by_key
    property required_setters using parsed_fy_py_files
    property required_property_setters_fy_py using parsed_fy_py_files
    property mixin_import_map using parsed_fy_py_files
    property files_to_generate using parsed_fy_py_file__and__folder_to_generate
fy"""

from pathlib import Path
from typing import Any

from fy_core.base.flow_base import FlowBase

from fy_library.flows.generate_and_save_flow_compose_files__using_parsed_fy_py_files_fy import (
    GenerateAndSaveFlowComposeFiles_UsingParsedFyPyFiles_Flow,
)
from fy_library.mixins.property.fy_file.folder_to_parse.using_setter import (
    FolderToParse_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.fy_file.project_root_folder.using_setter import (
    ProjectRootFolder_UsingSetter_PropertyMixin,
)
from fy_library.mixins.property.fy_py_file.fy_py_files_to_parse.using_files_discovery_fy import (
    FyPyFilesToParse_UsingFilesDiscovery_PropertyMixin,
)
from fy_library.mixins.property.imports_and_user_imports.mixin_import_map.using_parsed_fy_py_files_fy import (
    MixinImportMap_UsingParsedFyPyFiles_PropertyMixin,
)
from fy_library.mixins.property.mro.ordered_abstract_entities.using_parsed_fy_py_files_map_by_key_fy import (
    AbstractEntitiesOrderingIndex_UsingParsedFyPyFilesMapByKey_PropertyMixin,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files.using_fy_py_parser_fy import (
    ParsedFyPyFiles_UsingFyPyParser_PropertyMixin,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files_map_by_key.using_parsed_fy_py_files_fy import (
    ParsedFyPyFilesMapByKey_UsingParsedFyPyFiles_PropertyMixin,
)
from fy_library.mixins.property.parsed_fy_py.parsed_fy_py_files_with_own_abstract_mixin.using_parsed_fy_py_files_map_by_key_fy import (
    ParsedFyPyFilesWithOwnAbstractMixin_UsingParsedFyPyFilesMapByKey_PropertyMixin,
)
from fy_library.mixins.property.property_setters.required_property_setters_fy_py.using_parsed_fy_py_files_fy import (
    RequiredPropertySettersFyPy_UsingParsedFyPyFiles_PropertyMixin,
)
from fy_library.mixins.property.property_setters.required_setters.using_parsed_fy_py_files_fy import (
    RequiredSetters_UsingParsedFyPyFiles_PropertyMixin,
)

from fy_library.mixins.property.fy_file.folder_to_generate.using_setter import (
    FolderToGenerate_UsingSetter_PropertyMixin,
)

from fy_library.mixins.property.parsed_fy_py.files_to_generate.using_parsed_fy_py_file__and__folder_to_generate_fy import (
    FilesToGenerate_UsingParsedFyPyFile_And_FolderToGenerate_PropertyMixin,
)


# fy:start ===>>>
class FlowCompose_Main_Flow(
    # Property Mixins
    FolderToParse_UsingSetter_PropertyMixin,
    ProjectRootFolder_UsingSetter_PropertyMixin,
    FolderToGenerate_UsingSetter_PropertyMixin,
    FyPyFilesToParse_UsingFilesDiscovery_PropertyMixin,
    ParsedFyPyFiles_UsingFyPyParser_PropertyMixin,
    ParsedFyPyFilesMapByKey_UsingParsedFyPyFiles_PropertyMixin,
    ParsedFyPyFilesWithOwnAbstractMixin_UsingParsedFyPyFilesMapByKey_PropertyMixin,
    AbstractEntitiesOrderingIndex_UsingParsedFyPyFilesMapByKey_PropertyMixin,
    RequiredSetters_UsingParsedFyPyFiles_PropertyMixin,
    RequiredPropertySettersFyPy_UsingParsedFyPyFiles_PropertyMixin,
    MixinImportMap_UsingParsedFyPyFiles_PropertyMixin,
    FilesToGenerate_UsingParsedFyPyFile_And_FolderToGenerate_PropertyMixin,
    # Base
    FlowBase[None],
):
    def __init__(
        self,
        *args: Any,
        folder_to_parse: Path,
        project_root_folder: Path,
        folder_to_generate: Path,
        **kwargs: Any,
    ):
        self._folder_to_parse = folder_to_parse
        self._project_root_folder = project_root_folder
        self._folder_to_generate = folder_to_generate
        super().__init__(*args, **kwargs)

    def __call__(self) -> None:
        # fy:end <<<===
        GenerateAndSaveFlowComposeFiles_UsingParsedFyPyFiles_Flow(
            mixin_import_map=self._mixin_import_map,
            parsed_fy_py_files_map_by_key=self._parsed_fy_py_files_map_by_key,
            abstract_entities_ordering_index=self._abstract_entities_ordering_index,
            files_to_generate=self._files_to_generate,
        )()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
from typing import List
from fy_library.domain.mixin_models import AbstractMethodModel


property abstract_method_mixins: List[AbstractMethodModel]
fy"""

import abc
from typing import List

from fy_library.domain.mixin_models import AbstractMethodModel


# fy:start ===>>>
class AbstractMethodMixins_PropertyMixin_ABC(abc.ABC):
    @property
    @abc.abstractmethod
    def _abstract_method_mixins(self) -> List[AbstractMethodModel]:
        raise NotImplementedError()
        # fy:end <<<===



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""fy
property pre_marker_file_content: str using fy_py_file_parts:
    property fy_py_file_parts
fy"""

import abc
from functools import cached_property

from fy_library.mixins.property.fy_file.pre_marker_file_content.abc_fy import (
    PreMarkerFileContent_PropertyMixin_ABC,
)
from fy_library.mixins.property.fy_py_file.fy_py_file_parts.abc_fy import (
    FyPyFileParts_PropertyMixin_ABC,
)


# fy:start ===>>>
class PreMarkerFileContent_UsingFyPyFileParts_PropertyMixin(
    # Property Mixins
    FyPyFileParts_PropertyMixin_ABC,
    PreMarkerFileContent_PropertyMixin_ABC,
    abc.ABC,
):
    @cached_property
    def _pre_marker_file_content(self) -> str:
        # fy:end <<<===
        pre_marker_file_content = self._fy_py_file_parts.pre_marker_file_content
        return pre_marker_file_content
